<?php
namespace Controllers; 

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;

/**
 * Class to manipulate information and execute scripts for jobs in the database.
 * 
 * @author Grazielle Ferreira Antunes
 */
class AgendadorController extends  \Teknisa\Libs\Controller\CRUD{
    
    protected $crudService;
    protected $connection;
    protected $environment;
    protected $dateUtil;
    
    /**
     * Class constructor
     * 
     * Initialize an object's properties upon creation of the object.
     * 
     * @param $crudService  Object that allows to perform queries from datasource
     * @param object $environment - Allows access to session data
     * @param $connection Allows you to connect to the database
     * @param $dateUtil Class for handling dates
     * 
     * @return void
     * 
     * @access public
     * @author Grazielle Ferreira Antunes
     */
     
    public function __construct($crudService, $environment, $connection, $dateUtil) {
        $this->connection = $connection;
        $this->dateUtil = $dateUtil;
        $this->environment = $environment;
        parent::__construct($crudService, $environment);
    }
    
    
    /**
     * It manipulates data, calls the function to check if this job has already been scheduled,
     * calls the function to define the job execution frequency, prepares parameters 
     * for the scheduling script. Assembles and executes the task scheduling script in the database.
     * 
     * @param Request $request – Object that receives data from the caller.
     * @param Response $response –  Object that sends back data to the caller.
     * 
     * @access public
     * 
     * @author Grazielle Ferreira Antunes
     */ 
     
    public function agendar(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $dateTimeObject = null;
        
        $this->schedulingAlreadyExists($row);
        if ($row['LANGUAGE'] == 'en_us') {
            $dateTimeObject = $this->dateUtil->getDataDeString($row["PROXIMA_EXEC"], 'm/d/Y h:i A', false);
        } else {
            $dateTimeObject = $this->dateUtil->getDataDeString($row["PROXIMA_EXEC"], 'd/m/Y H:i', false);
        }
        $proximaExec = $dateTimeObject->format('Y-m-d H:i:s');
        $timezone = $dateTimeObject->getTimezone()->getName();
        
        $frequency = $this->defineFrequency($dateTimeObject, $row['FREQUENCIA']);

        if(\Zeedhi\Framework\Util\Functions::arrayKeyExists('DTLIMPEZA', $row)){
            $dtlimpeza = $row['DTLIMPEZA'];
        } else {
            $dtlimpeza = NULL;
        }

        $params = array(
            'P_NMJOB'          => $row["NMJOB"],
            'P_PROXIMA_EXEC'   => $proximaExec,
            'P_DTLIMPEZA'      => $row["DTLIMPEZA"],
            'P_INTERVAL'       => $frequency
        );
        
        try{
            $stmt = $this->connection->prepare(
                "
                    DECLARE
                        V_WHAT  VARCHAR(4000);
                        V_COMMENTS VARCHAR(4000);
    
                    BEGIN
                        
                        FOR C_JOBS IN (SELECT JOB
                                            FROM USER_JOBS
                                        WHERE WHAT LIKE '%'||:P_NMJOB||'%') LOOP
                            DBMS_JOB.REMOVE(C_JOBS.JOB);
                        END LOOP;
    
                        UPDATE AGENDAJOB
                             SET DTLIMPEZA = :P_DTLIMPEZA
                            WHERE NMJOB = :P_NMJOB;
    
                        SELECT MAX(DSWHAT), MAX(NMROTINAEN) INTO V_WHAT, V_COMMENTS FROM AGENDAJOB WHERE NMJOB = :P_NMJOB; 
                        DBMS_SCHEDULER.CREATE_JOB (
                            job_name => :P_NMJOB || '_JOB',
                            job_type => 'PLSQL_BLOCK',
                            job_action => V_WHAT,
                            start_date => TO_DATE(:P_PROXIMA_EXEC, 'YYYY-MM-DD HH24:MI:SS'),
                            repeat_interval => :P_INTERVAL,
                            end_date => NULL,
                            enabled => TRUE,
                            auto_drop => FALSE,
                            comments => V_COMMENTS);
                        COMMIT;
                    END;
                ");
            $stmt->execute($params);
        }catch(\Exception $ex){
            $response->setError(new Error("Nao foi possivel processar os dados enviados   --- " . $ex->getMessage(), 500));
        }

    }
    
    /**
     * Check in the database if the chosen job is already scheduled, if yes, call the functions to disable the job and later to drop the job
     * 
     * @param $row – Current row / Object that receives data from the caller.
     * 
     * @access private
     * 
     * @author Grazielle Ferreira Antunes
     */
    private function schedulingAlreadyExists($row){
        $params = array(
            'P_NMJOB'          => $row["NMJOB"]
        );
        $stmt = $this->connection->prepare(
                "
                  SELECT * FROM USER_SCHEDULER_JOBS WHERE JOB_NAME = :P_NMJOB || '_JOB'
                ");
        $stmt->execute($params);
        $result = $stmt->fetchAll();
        
        if(!empty($result)){
            $row['JOB'] = $row['NMJOB'] . '_JOB';
            $row['STATUS'] = 'TRUE';
            $this->disableOrEnable($row);
            $this->dropJob($row);
        }
    }
    
    /**
     * According to the user's choice, mount the frequency in the format used by DBMS_SCHEDULER to be recorded 
     * in the database and returns it to the scheduling function.
     * 
     * @param $dateTimeObject – Object DateTime.
     * @param $frequencyChosen – Frequency chosen by the user.
     * 
     * return $frequency - String
     * 
     * @access private
     * 
     * @author Grazielle Ferreira Antunes
     */ 
    private function defineFrequency($dateTimeObject, $frequencyChosen) {
        
        $day = $dateTimeObject->format('d');
        $hour = $dateTimeObject->format('H');
        $minutes = $dateTimeObject->format('i');
        $dayweek = strtoupper($dateTimeObject->format('D'));
        
        switch($frequencyChosen){
            case 'TRINTAMINUTOS' :
                $frequency = 'FREQ=MINUTELY;INTERVAL=30;';
                break;
            case 'UMAHORA' :
                $frequency = 'FREQ=HOURLY;INTERVAL=1;';
                break;
            case 'DUASHORAS' :
                $frequency = 'FREQ=HOURLY;INTERVAL=2;';
                break;
            case 'DIARIO':
                $frequency = 'FREQ=DAILY';
                break;
            case 'SEMANAL':
                $frequency = 'FREQ=WEEKLY';
                break;
            case 'MENSAL':
                $frequency = 'FREQ=MONTHLY';
                break;
        };
        return $frequency;
    }
    
    /**
     *Prepare the parameters and execute the script of the chosen routine immediately.
     * 
     * @param Request $request – Object that receives data from the caller.
     * @param Response $response –  Object that sends back data to the caller.
     * 
     * @access public
     * 
     * @author Grazielle Ferreira Antunes
     */ 
    public function executar(DTO\Request\Row $request, DTO\Response $response) {
        
        $row  = $request->getRow();
        
        $params = array(
            'P_NRAGENDAEVENTOS' => $row["NRAGENDAEVENTOS"]
        );
        
        try {
            $stmt = $this->connection->prepare(
            "DECLARE V_DSWHAT VARCHAR2(4000);
            BEGIN
               SELECT DSWHAT INTO V_DSWHAT FROM AGENDAJOB WHERE NRAGENDAEVENTOS = :P_NRAGENDAEVENTOS;
               EXECUTE IMMEDIATE V_DSWHAT;
            END;"
            );
            $stmt->execute($params);


        } catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 1));
        }

    }
    
    /**
     * Receives the object with data from the current row and calls the function to enable or disable the job
     * 
     * @param Request $request – Object that receives data from the caller.
     * @param Response $response –  Object that sends back data to the caller.
     * 
     * @access public
     * 
     * @author Grazielle Ferreira Antunes
     */ 
    public function habilitaDesabilitaJob(DTO\Request\Row $request, DTO\Response $response) {
        $row  = $request->getRow();
        $this->disableOrEnable($row);
    }
        
    /**
     * Assemble the script to disable or enable a job according to the button pressed by the user and execute in the database.
     * 
     * @param $row – Current row / Object that receives data from the caller.
     * 
     * @access private
     * 
     * @author Grazielle Ferreira Antunes
     */ 
    private function disableOrEnable($row) {
        
        if($row['STATUS'] == 'TRUE') {
            
            $comand =  "
                BEGIN 
                    UPDATE AGENDAJOB A
                        SET A.DTPROCESSO = (SELECT J.NEXT_RUN_DATE FROM USER_SCHEDULER_JOBS J WHERE J.JOB_NAME = :P_JOB)
                        WHERE EXISTS (SELECT JOB_NAME FROM USER_SCHEDULER_JOBS J WHERE J.JOB_NAME = A.NMJOB);
                     BEGIN                         
                        DBMS_SCHEDULER.STOP_JOB(:P_JOB);
                    EXCEPTION
                        WHEN OTHERS THEN
                            NULL;
                    END;
                    DBMS_SCHEDULER.DISABLE(:P_JOB);
                END; ";
        } else {
            
            $comand = "BEGIN
                            DBMS_SCHEDULER.ENABLE(:P_JOB);
                        END;";
        }
        $params = array(
            'P_JOB' => $row["JOB"]
        );
        
        try {
            $stmt = $this->connection->prepare($comand);
            $stmt->execute($params);

        } catch(\Exception $ex){
            $response->setError(new Error("Nao foi possivel processar os dados enviados   --- " . $ex->getMessage(), 500));
        }

    }
    
    /**
     * Receives the object with data from the current line and calls the function to delete the job.
     * 
     * @param Request $request – Object that receives data from the caller.
     * @param Response $response –  Object that sends back data to the caller.
     * 
     * @access public
     * 
     * @author Grazielle Ferreira Antunes
     */ 
    public function removeJob(DTO\Request\Row $request, DTO\Response $response) {
        $row  = $request->getRow();
        $this->dropJob($row);
    }

    /**
     * Delete the job schedule in the database
     * 
     * @param $row – Current row / Object that receives data from the caller.
     * 
     * @access private
     * 
     * @author Grazielle Ferreira Antunes
     */ 
    public function dropJob($row) {
        
        $params = array(
            'P_JOB' => $row["JOB"]
        );
        
        try {
            $stmt = $this->connection->prepare("
                BEGIN
                    DBMS_SCHEDULER.DROP_JOB(:P_JOB);
                    COMMIT;
                END;
            ");

            $stmt->execute($params);

        } catch(\Exception $e) {
            $response->setError(new Error("Nao foi possivel processar os dados enviados   --- " . $e->getMessage(), 500));
        }
    }

}